import { createContext, useContext, ReactNode, useEffect } from 'react';
import { useForm, FormProvider, SubmitHandler } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { z } from 'zod';
import { CourseDto, CourseLevel, CourseStatus } from '@nicestack/common';
import { api, useCourse } from '@nicestack/client';
import toast from 'react-hot-toast';
import { useNavigate } from 'react-router-dom';
// 定义课程表单验证 Schema
const courseSchema = z.object({
    title: z.string().min(1, '课程标题不能为空'),
    subTitle: z.string().nullish(),
    description: z.string().nullish(),
    thumbnail: z.string().url().nullish(),
    level: z.nativeEnum(CourseLevel),
    requirements: z.array(z.string()).nullish(),
    objectives: z.array(z.string()).nullish(),
    skills: z.array(z.string()).nullish(),
    audiences: z.array(z.string()).nullish(),
    status: z.nativeEnum(CourseStatus),
});
export type CourseFormData = z.infer<typeof courseSchema>;
interface CourseEditorContextType {
    onSubmit: SubmitHandler<CourseFormData>;
    editId?: string; // 添加 editId
    course?: CourseDto
}
interface CourseFormProviderProps {
    children: ReactNode;
    editId?: string; // 添加 editId 参数
}
const CourseEditorContext = createContext<CourseEditorContextType | null>(null);
export function CourseFormProvider({ children, editId }: CourseFormProviderProps) {
    const { create, update } = useCourse()
    const { data: course }: { data: CourseDto } = api.course.findFirst.useQuery({ where: { id: editId } }, { enabled: Boolean(editId) })
    const methods = useForm<CourseFormData>({
        resolver: zodResolver(courseSchema),
        defaultValues: {
            status: CourseStatus.DRAFT,
            level: CourseLevel.BEGINNER,
            requirements: [],
            objectives: [],
            skills: [],
            audiences: [],
        },
    });
    const navigate = useNavigate()
    useEffect(() => {
        if (course) {
            // 只选择表单需要的字段
            const formData = {
                title: course.title,
                subTitle: course.subTitle,
                description: course.description,
                thumbnail: course.thumbnail,
                level: course.level,
                requirements: course.requirements,
                objectives: course.objectives,
                skills: course.skills,
                audiences: course.audiences,
                status: course.status,
            };
            methods.reset(formData as any);
        }
    }, [course, methods]);
    const onSubmit: SubmitHandler<CourseFormData> = async (data: CourseFormData) => {
        try {
            if (editId) {
                await update.mutateAsync({
                    where: { id: editId },
                    data: {
                        ...data
                    }
                })
                toast.success('课程更新成功！');
            } else {
                const result = await create.mutateAsync({
                    data: {
                        ...data
                    }
                })
                console.log(`/course/${result.id}/manage`)
                navigate(`/course/${result.id}/manage`, { replace: true })
                toast.success('课程创建成功！');
            }

        } catch (error) {
            console.error('Error submitting form:', error);
            toast.error('操作失败，请重试！');
        }
    };

    return (
        <CourseEditorContext.Provider value={{ onSubmit, editId, course }}>
            <FormProvider {...methods}>
                {children}
            </FormProvider>
        </CourseEditorContext.Provider>
    );
}

export const useCourseEditor = () => {
    const context = useContext(CourseEditorContext);
    if (!context) {
        throw new Error('useCourseEditor must be used within CourseFormProvider');
    }
    return context;
};